package com.neusoft.smart.pos.inventory.persistent.mapper;

import com.neusoft.smart.pos.inventory.persistent.domain.InventoryStoreHistory;
import com.neusoft.smart.pos.inventory.persistent.domain.InventoryStoreHistoryExample;
import com.neusoft.smart.pos.inventory.persistent.domain.InventoryStoreTotal;
import com.neusoft.smart.pos.inventory.persistent.domain.InventoryStoreTotalExample;
import com.neusoft.smart.pos.inventory.persistent.domain.InventoryTotalAmount;
import org.apache.ibatis.annotations.Mapper;

import java.util.List;

@Mapper
public interface InventoryStoreMapper {
    
    int countTotalByExample(InventoryStoreTotalExample example);
    
    List<InventoryStoreTotal> selectTotalByExample(InventoryStoreTotalExample example);
    
    List<InventoryStoreTotal> selectTotalByShopId(Integer shopId);
    
    InventoryTotalAmount selectInventoryTotalAmount(Integer shopId);
    
    List<InventoryStoreHistory> selectHistoryByExample(InventoryStoreHistoryExample example);
    
    int countHistoryByExample(InventoryStoreHistoryExample example);
}